unit EXIF;
interface
uses SysUtils,Math;

type EXIFOutputHandler=procedure(TagID,Value:integer; const TagName,Content:string);

function TagIDToStr(Tag:integer):string;
function EXIFParse(const FileName:string; OutputHandler:EXIFOutputHandler):boolean;

const EXIF_TAG_NewSubFileType=$00fe;
      EXIF_TAG_ImageWidth=$0100;
      EXIF_TAG_ImageLength=$0101;
      EXIF_TAG_BitsPerSample=$0102;
      EXIF_TAG_Compression=$0103;
      EXIF_TAG_PhotometricInterpretation=$0106;
      EXIF_TAG_FillOrder=$010a;
      EXIF_TAG_DocumentName=$010d;
      EXIF_TAG_ImageDescription=$010e;
      EXIF_TAG_Make=$010f;
      EXIF_TAG_Model=$0110;
      EXIF_TAG_StripOffsets=$0111;
      EXIF_TAG_Orientation=$0112;
      EXIF_TAG_SamplesPerPixel=$0115;
      EXIF_TAG_RowsPerStrip=$0116;
      EXIF_TAG_StripByteCounts=$0117;
      EXIF_TAG_XResolution=$011a;
      EXIF_TAG_YResolution=$011b;
      EXIF_TAG_PlanarConfiguration=$011c;
      EXIF_TAG_ResolutionUnit=$0128;
      EXIF_TAG_TransferFunction=$012d;
      EXIF_TAG_Software=$0131;
      EXIF_TAG_DateTime=$0132;
      EXIF_TAG_Artist=$013b;
      EXIF_TAG_WhitePoint=$013e;
      EXIF_TAG_PrimaryChromaticities=$013f;
      EXIF_TAG_TileWidth=$0142;
      EXIF_TAG_TileLength=$0143;
      EXIF_TAG_TileOffsets=$0144;
      EXIF_TAG_TileByteCounts=$0145;
      EXIF_TAG_SubIFDs=$014a;
      EXIF_TAG_TransferRange=$0156;
      EXIF_TAG_JPEGTables=$015b;
      EXIF_TAG_JPEGInterchangeFormat=$0201;
      EXIF_TAG_JPEGInterchangeFormatLength=$0202;
      EXIF_TAG_YCbCrCoefficients=$0211;
      EXIF_TAG_YCbCrSubSampling=$0212;
      EXIF_TAG_YCbCrPositioning=$0213;
      EXIF_TAG_ReferenceBlackWhite=$0214;
      EXIF_TAG_CFARepeatPatternDim=$828d;
      EXIF_TAG_CFAPattern=$828e;
      EXIF_TAG_BatteryLevel=$828f;
      EXIF_TAG_Copyright=$8298;
      EXIF_TAG_ExposureTime=$829a;
      EXIF_TAG_FNumber=$829d;
      EXIF_TAG_IPTC_NAA=$83bb;
      EXIF_TAG_InterColorProfile=$8773;
      EXIF_TAG_ExposureProgram=$8822;
      EXIF_TAG_SpectralSensitivity=$8824;
      EXIF_TAG_GPSInfo=$8825;
      EXIF_TAG_ISOSpeedRatings=$8827;
      EXIF_TAG_OECF=$8828;
      EXIF_TAG_Interlace=$8829;
      EXIF_TAG_TimeZoneOffset=$882a;
      EXIF_TAG_SelfTimerMode=$882b;
      EXIF_TAG_ExifOffset=$8769;
      EXIF_TAG_ExifVersion=$9000;
      EXIF_TAG_DateTimeOriginal=$9003;
      EXIF_TAG_DateTimeDigitized=$9004;
      EXIF_TAG_ComponentsConfiguration=$9101;
      EXIF_TAG_CompressedBitsPerPixel=$9102;
      EXIF_TAG_ShutterSpeedValue=$9201;
      EXIF_TAG_ApertureValue=$9202;
      EXIF_TAG_BrightnessValue=$9203;
      EXIF_TAG_ExposureBiasValue=$9204;
      EXIF_TAG_MaxApertureValue=$9205;
      EXIF_TAG_SubjectDistance=$9206;
      EXIF_TAG_MeteringMode=$9207;
      EXIF_TAG_LightSource=$9208;
      EXIF_TAG_Flash=$9209;
      EXIF_TAG_FocalLength=$920a;
      EXIF_TAG_FlashEnergy=$a20b;
      EXIF_TAG_SpatialFrequencyResponse=$a20c;
      EXIF_TAG_Noise=$920d;
      EXIF_TAG_FocalPlaneXResolution=$a20e;
      EXIF_TAG_FocalPlaneYResolution=$a20f;
      EXIF_TAG_FocalPlaneResolutionUnit=$a210;
      EXIF_TAG_ImageNumber=$9211;
      EXIF_TAG_SecurityClassification=$9212;
      EXIF_TAG_ImageHistory=$9213;
      EXIF_TAG_SubjectLocation=$a214;
      EXIF_TAG_ExposureIndex=$a215;
      EXIF_TAG_TIFF_EPStandardID=$9216;
      EXIF_TAG_SensingMethod=$a217;
      EXIF_TAG_MakerNote=$927c;
      EXIF_TAG_FileSource=$a300;
      EXIF_TAG_SceneType=$a301;
      EXIF_TAG_ColorSpace=$a001;
      EXIF_TAG_ExifImageWidth=$a002;
      EXIF_TAG_ExifImageHeight=$a003;
      EXIF_TAG_Interoperability_IFD_Pointer=$a005;

const EXIF_TAGTYPE_BYTE=1;
      EXIF_TAGTYPE_ASCII=2;
      EXIF_TAGTYPE_SHORT=3;
      EXIF_TAGTYPE_LONG=4;
      EXIF_TAGTYPE_RATIONAL=5;
      EXIF_TAGTYPE_UNDEFINED=7;
      EXIF_TAGTYPE_SLONG=9;
      EXIF_TAGTYPE_SRATIONAL=10;

////////////////////////////////////////////////////////////////////////////////
implementation
const EXIFBufferSize=4096;
const hex:array[0..15]of char='0123456789ABCDEF';

function TagIDToStr(Tag:integer):string;
begin
  case Tag of
    $00fe:Result:='NewSubFileType';
    $0100:Result:='ImageWidth';
    $0101:Result:='ImageLength';
    $0102:Result:='BitsPerSample';
    $0103:Result:='Compression';
    $0106:Result:='PhotometricInterpretation';
    $010a:Result:='FillOrder';
    $010d:Result:='DocumentName';
    $010e:Result:='ImageDescription';
    $010f:Result:='Make';
    $0110:Result:='Model';
    $0111:Result:='StripOffsets';
    $0112:Result:='Orientation';
    $0115:Result:='SamplesPerPixel';
    $0116:Result:='RowsPerStrip';
    $0117:Result:='StripByteCounts';
    $011a:Result:='XResolution';
    $011b:Result:='YResolution';
    $011c:Result:='PlanarConfiguration';
    $0128:Result:='ResolutionUnit';
    $012d:Result:='TransferFunction';
    $0131:Result:='Software';
    $0132:Result:='DateTime';
    $013b:Result:='Artist';
    $013e:Result:='WhitePoint';
    $013f:Result:='PrimaryChromaticities';
    $0142:Result:='TileWidth';
    $0143:Result:='TileLength';
    $0144:Result:='TileOffsets';
    $0145:Result:='TileByteCounts';
    $014a:Result:='SubIFDs';
    $0156:Result:='TransferRange';
    $015b:Result:='JPEGTables';
    $0201:Result:='JPEGInterchangeFormat';
    $0202:Result:='JPEGInterchangeFormatLength';
    $0211:Result:='YCbCrCoefficients';
    $0212:Result:='YCbCrSubSampling';
    $0213:Result:='YCbCrPositioning';
    $0214:Result:='ReferenceBlackWhite';
    $828d:Result:='CFARepeatPatternDim';
    $828e:Result:='CFAPattern';
    $828f:Result:='BatteryLevel';
    $8298:Result:='Copyright';
    $829a:Result:='ExposureTime';
    $829d:Result:='FNumber';
    $83bb:Result:='IPTC_NAA';
    $8773:Result:='InterColorProfile';
    $8822:Result:='ExposureProgram';
    $8824:Result:='SpectralSensitivity';
    $8825:Result:='GPSInfo';
    $8827:Result:='ISOSpeedRatings';
    $8828:Result:='OECF';
    $8829:Result:='Interlace';
    $882a:Result:='TimeZoneOffset';
    $882b:Result:='SelfTimerMode';
    $8769:Result:='ExifOffset';
    $9000:Result:='ExifVersion';
    $9003:Result:='DateTimeOriginal';
    $9004:Result:='DateTimeDigitized';
    $9101:Result:='ComponentsConfiguration';
    $9102:Result:='CompressedBitsPerPixel';
    $9201:Result:='ShutterSpeedValue';
    $9202:Result:='ApertureValue';
    $9203:Result:='BrightnessValue';
    $9204:Result:='ExposureBiasValue';
    $9205:Result:='MaxApertureValue';
    $9206:Result:='SubjectDistance';
    $9207:Result:='MeteringMode';
    $9208:Result:='LightSource';
    $9209:Result:='Flash';
    $920a:Result:='FocalLength';
    $a20b:Result:='FlashEnergy';
    $a20c:Result:='SpatialFrequencyResponse';
    $920d:Result:='Noise';
    $a20e:Result:='FocalPlaneXResolution';
    $a20f:Result:='FocalPlaneYResolution';
    $a210:Result:='FocalPlaneResolutionUnit';
    $9211:Result:='ImageNumber';
    $9212:Result:='SecurityClassification';
    $9213:Result:='ImageHistory';
    $a214:Result:='SubjectLocation';
    $a215:Result:='ExposureIndex';
    $9216:Result:='TIFF_EPStandardID';
    $a217:Result:='SensingMethod';
    $927c:Result:='MakerNote';
    $a300:Result:='FileSource';
    $a301:Result:='SceneType';
    $a001:Result:='ColorSpace';
    $a002:Result:='ExifImageWidth';
    $a003:Result:='ExifImageHeight';
    $a005:Result:='Interoperability_IFD_Pointer';
  else
    Result:='Tag0x'+
            hex[(Tag AND $F000) SHR 12]+hex[(Tag AND $0F00) SHR  8]+
            hex[(Tag AND $00F0) SHR  4]+hex[(Tag AND $000F)       ];
  end;
end;

////////////////////////////////////////////////////////////////////////////////
function EXIFParse(const FileName:string; OutputHandler:EXIFOutputHandler):boolean;
var EXIFBuffer:array[0..EXIFBufferSize-1]of byte;
    EXIFPtr:integer;
    FortyTwo:integer;
    Base:integer;
  function GetWord(Index:integer):word;
  begin
    if FortyTwo=$002A
      then Result:=EXIFBuffer[Index] OR (EXIFBuffer[Index+1] SHL 8)
      else Result:=(EXIFBuffer[Index] SHL 8) OR EXIFBuffer[Index+1];
  end;
  function GetLong(Index:integer):integer;
  begin
    if FortyTwo=$002A
      then Result:=GetWord(Index) OR (GetWord(Index+2) SHL 16)
      else Result:=(GetWord(Index) SHL 16) OR GetWord(Index+2);
  end;

  procedure ProcessEntry(Index:integer);
  var Tag,TType,Len,Value,Offset:integer;
      TextValue:PChar;
      i:integer;
      IsTerminated:boolean;
      Result:string;
      f:double;
    procedure UnknownValue;
    begin
      Result:='Unknown (0x'+
        hex[(Value AND $F0000000) SHR 28]+hex[(Value AND $0F000000) SHR 24]+
        hex[(Value AND $00F00000) SHR 20]+hex[(Value AND $000F0000) SHR 16]+
        hex[(Value AND $0000F000) SHR 12]+hex[(Value AND $00000F00) SHR  8]+
        hex[(Value AND $000000F0) SHR  4]+hex[ Value AND $0000000F        ]+
      ')';
    end;
  begin
    Tag:=GetWord(Index);
    TType:=GetWord(Index+2);
    Len:=GetLong(Index+4);
    Value:=GetLong(Index+8);
    Offset:=Base+Value+10;
    if Tag=$8769 then begin
      EXIFPtr:=Offset;
      exit;
    end;
    Result:='';
    case Tag of
      $0103:case Value of
              1:Result:='Uncompressed';
              6:Result:='JPEG';
            else UnknownValue; end;
      $0106:case Value of
              2:Result:='RGB';
              6:Result:='YCbCr';
            else UnknownValue; end;
      $0112:case Value of
              1:Result:='Landscape';
              2:Result:='Landscape, mirrored';
              3:Result:='Landscape, rotated';
              4:Result:='Landscape, flipped';
              5:Result:='Portrait, mirrored';
              6:Result:='Portrait';
              7:Result:='Portrait, flipped';
              8:Result:='Portrait, rotated';
            else UnknownValue; end;
      $011C:case Value of
              1:Result:='Chunky';
              2:Result:='Planar';
            else UnknownValue; end;
      $0128:case Value of
              2:Result:='in';
              3:Result:='cm';
            else UnknownValue; end;
      $0213:case Value of
              1:Result:='Centered';
              2:Result:='Co-Sited';
            else UnknownValue; end;
      $8822:case Value of
              0:Result:='Not defined';
              1:Result:='Manual';
              2:Result:='Normal program';
              3:Result:='Aperture priority';
              4:Result:='Shutter priority';
              5:Result:='Creative program (biased toward depth of field)';
              6:Result:='Action program (biased toward fast shutter speed)';
              7:Result:='Portrait mode (for closeup photos with the background out of focus)';
              8:Result:='Landscape mode (for landscape photos with the background in focus)';
            else UnknownValue; end;
      $9207:case Value of
              1:Result:='Average';
              2:Result:='CenterWeightedAverage';
              3:Result:='Spot';
              4:Result:='MultiSpot';
              5:Result:='Pattern';
              6:Result:='Partial';
            else UnknownValue; end;
      $9208:case Value of
              1:Result:='Daylight';
              2:Result:='Fluorescent';
              3:Result:='Tungsten';
              17:Result:='Standard light A';
              18:Result:='Standard light B';
              19:Result:='Standard light C';
              20:Result:='D55';
              21:Result:='D65';
              22:Result:='D75';
            else UnknownValue; end;
      $9209:case (Value AND 31) of
              0:Result:='no';
              1:Result:='fired';
              5:Result:='fired, no return sensed';
              7:Result:='fired, return sensed';
              9:Result:='fill fired';
              13:Result:='fill fired, no return sensed';
              15:Result:='fill fired, return sensed';
              16:Result:='off';
              24:Result:='auto off';
              25:Result:='auto fired';
              29:Result:='auto fired, no return sensed';
              31:Result:='auto fired, return sensed';
            else UnknownValue; end;
      $9210:case Value of
              1:Result:='in';
              2:Result:='m';
              3:Result:='cm';
              4:Result:='mm';
              5:Result:='m';
            else UnknownValue; end;
      $a217:case Value of
              1:Result:='Not defined';
              2:Result:='One-chip color area sensor';
              3:Result:='Two-chip color area sensor';
              4:Result:='Three-chip color area sensor';
              5:Result:='Color sequential area sensor';
              7:Result:='Trilinear sensor';
              8:Result:='Color sequential linear sensor';
            else UnknownValue; end;
      $a300:case Value of
              3:Result:='Digital Still Camera';
            else UnknownValue; end;
      $a301:case Value of
              1:Result:='Directly Photographed Scene';
            else UnknownValue; end;
      $9000:begin
              SetLength(Result,4);
              Result[1]:=char(EXIFBuffer[Index+8]);
              Result[2]:=char(EXIFBuffer[Index+9]);
              Result[3]:=char(EXIFBuffer[Index+10]);
              Result[4]:=char(EXIFBuffer[Index+11]);
            end;
    else
      case TType of
        EXIF_TAGTYPE_BYTE:
          Result:=IntToStr(Value AND $FF);
        EXIF_TAGTYPE_ASCII: begin
          TextValue:=@EXIFBuffer[Offset];
          IsTerminated:=false;
          for i:=0 to Len do
            if TextValue[i]=#0 then begin
              IsTerminated:=true;
              break;
            end;
          if not IsTerminated then exit;
          Result:=TextValue;
        end;
        EXIF_TAGTYPE_SHORT:
          Result:=IntToStr(Value AND $FFFF);
        EXIF_TAGTYPE_LONG,EXIF_TAGTYPE_SLONG:
          Result:=IntToStr(Value);
        EXIF_TAGTYPE_RATIONAL,EXIF_TAGTYPE_SRATIONAL: begin
          f:=GetLong(Offset)/GetLong(Offset+4);
          case Tag of
            EXIF_TAG_ExposureTime:
              if f<1.0 then Result:=Format('1/%.0f s',[1.0/f])
                       else Result:=Format('%.1f s',[f]);
            EXIF_TAG_ShutterSpeedValue: begin
              f:=Power(0.5,f);
              if f<1.0 then Result:=Format('1/%.0f s',[1.0/f])
                       else Result:=Format('%.1f s',[f]);
            end;
            EXIF_TAG_FNumber: Result:=Format('F%.1f',[f]);
            EXIF_TAG_FocalLength: Result:=Format('%.1f mm',[f]);
          else
            Result:=Format('%.2f (%d/%d)',[f,GetLong(Offset),GetLong(Offset+4)]);
          end;
        end;
        EXIF_TAGTYPE_UNDEFINED:
          Result:='0x'+
            hex[(Value AND $F0000000) SHR 28]+hex[(Value AND $0F000000) SHR 24]+
            hex[(Value AND $00F00000) SHR 20]+hex[(Value AND $000F0000) SHR 16]+
            hex[(Value AND $0000F000) SHR 12]+hex[(Value AND $00000F00) SHR  8]+
            hex[(Value AND $000000F0) SHR  4]+hex[ Value AND $0000000F        ];
      else UnknownValue; end;
    end;
    OutputHandler(Tag,Value,TagIDToStr(Tag),Result);
  end;
var f:file;
    Index:integer;
    OldFileMode:byte;
begin
  Result:=false;
  AssignFile(f,FileName);
  OldFileMode:=FileMode; FileMode:=fmOpenRead;
  {$I-} Reset(f,1); {$I+}
  FileMode:=OldFileMode;
  if IOresult<>0 then exit;
  EXIFBuffer[0]:=0;
  BlockRead(f,EXIFBuffer,EXIFBufferSize,Index);
  CloseFile(f);
  if (EXIFBuffer[0]<>$FF) OR (EXIFBuffer[1]<>$D8) then exit;
  Base:=2;
  if (EXIFBuffer[2]=$FF) AND (EXIFBuffer[3]=$E0) then
    inc(Base,((EXIFBuffer[4] SHL 8) OR EXIFBuffer[5])+2);
  if (EXIFBuffer[Base+0]<>$FF) OR (EXIFBuffer[Base+1]<>$E1) OR
     (EXIFBuffer[Base+4]<>$45) OR (EXIFBuffer[Base+5]<>$78) OR
     (EXIFBuffer[Base+6]<>$69) OR (EXIFBuffer[Base+7]<>$66) OR
     (EXIFBuffer[Base+8]<>$00) OR (EXIFBuffer[Base+9]<>$00) then exit;
  FortyTwo:=word((@EXIFBuffer[Base+12])^);
  if (FortyTwo<>$2A00) AND (FortyTwo<>$002A) then exit;
  Index:=20; EXIFPtr:=0;
  repeat
    ProcessEntry(Base+Index);
    inc(Index,12);
  until (Index>=152) OR (EXIFPtr<>0);
  Index:=EXIFPtr+2;
  while (Index<EXIFBufferSize-12) AND (GetWord(Index)<>0) do begin
    ProcessEntry(Index);
    inc(Index,12);
  end;
  Result:=true;
end;

end.

