unit EXIFView;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Buttons, StdCtrls, EXIF, Menus;

type
  TEXIFForm = class(TForm)
    LCamera: TLabel;
    LDateTime: TLabel;
    LSettings: TLabel;
    BMenu: TSpeedButton;
    Info: TListBox;
    Popup: TPopupMenu;
    MHideUnknownTags: TMenuItem;
    MSortTags: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure BMenuClick(Sender: TObject);
    procedure MHideUnknownTagsClick(Sender: TObject);
    procedure PopupPopup(Sender: TObject);
    procedure MSortTagsClick(Sender: TObject);
    procedure InfoDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  private
    { Private declarations }
    iDate,iAperture,iExposure,iFocus,iISO,iFlash:string;
    Spacing:integer;
  public
    { Public declarations }
    HideUnknownTags:boolean;
    procedure ReadImage;
  end;

var
  EXIFForm: TEXIFForm;

implementation

uses Main;

{$R *.dfm}
{$R ExifLogo.res}

procedure TEXIFForm.FormCreate(Sender: TObject);
begin
  Font:=MainForm.Font;
  Top:=0; Left:=Screen.Width-Width;
  AutoScroll:=false;
  BMenu.Glyph.LoadFromResourceName(MainInstance,'EXIFLOGO');

  HideUnknownTags:=true;

  ReadImage();
end;

procedure TEXIFForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if upcase(Key)='E' then Visible:=not(Visible);
end;

procedure EXIFParseCallback(TagID,Value:integer; const TagName,Content:string);
var W:integer;
begin with EXIFForm do begin
  case TagID of
    EXIF_TAG_Model: LCamera.Caption:=Content;
    EXIF_TAG_DateTime: iDate:=Trim(Content);
    EXIF_TAG_DateTimeOriginal: if iDate='' then iDate:=Content;
    EXIF_TAG_FNumber: iAperture:=Content;
    EXIF_TAG_ExposureTime: iExposure:=Content;
    EXIF_TAG_ShutterSpeedValue: if iExposure='' then iExposure:=Content;
    EXIF_TAG_FocalLength: iFocus:=Content;
    EXIF_TAG_ISOSpeedRatings: if Value>0 then iISO:='ISO '+Content;
    EXIF_TAG_Flash: case (Value AND 31) of
                      1,5,7,9,13,15: iFlash:='Blitz';
                      25,29,31: iFlash:='Auto-Blitz';
                    end;
  end;
  if HideUnknownTags AND (copy(TagName,1,5)='Tag0x') then exit;
  W:=Canvas.TextWidth(TagName);
  if W>Spacing then Spacing:=W;
  Info.Items.Add(TagName+'='+Content)
end; end;

procedure TEXIFForm.ReadImage;
var s:string;
begin
  LDateTime.Caption:=ExtractFileName(MainForm.CurrentImagePath);
  LCamera.Caption:='';
  LSettings.Caption:='';
  Info.Clear;
  if MainForm.CurrentImagePath='' then begin
    LDateTime.Caption:='Keine Datei';
    exit;
  end;
  iDate:=''; iAperture:=''; iExposure:=''; iFocus:=''; iISO:=''; iFlash:='';
  Spacing:=0;
  if not EXIFParse(MainForm.CurrentImagePath,EXIFParseCallback) then begin
    LDateTime.Caption:=LDateTime.Caption+': Keine EXIF-Informationen verfgbar';
    exit;
  end;
  if iDate<>'' then LDateTime.Caption:=LDateTime.Caption+': '+
                    copy(iDate,9,2)+'.'+copy(iDate,6,2)+'.'+copy(iDate,1,4)+
                    copy(iDate,11,9);
  s:='';
  if iAperture<>'' then s:=s+iAperture+', ';
  if iExposure<>'' then s:=s+iExposure+', ';
  if iFocus<>'' then s:=s+iFocus+', ';
  if iISO<>'' then s:=s+iISO+', ';
  if iFlash<>'' then s:=s+iFlash+', ';
  if s<>'' then LSettings.Caption:=copy(s,1,length(s)-2);
  Repaint;
end;

procedure TEXIFForm.BMenuClick(Sender: TObject);
var PopupPoint:TPoint;
begin
  PopupPoint:=ClientToScreen(Point(BMenu.Width,0));
  Popup.Popup(PopupPoint.X,PopupPoint.Y);
end;

procedure TEXIFForm.MHideUnknownTagsClick(Sender: TObject);
begin
  HideUnknownTags:=not(HideUnknownTags);
  ReadImage();
end;

procedure TEXIFForm.PopupPopup(Sender: TObject);
begin
  MHideUnknownTags.Checked:=HideUnknownTags;
  MSortTags.Checked:=Info.Sorted;
end;

procedure TEXIFForm.MSortTagsClick(Sender: TObject);
begin
  Info.Sorted:=not(Info.Sorted);
  ReadImage();
end;

procedure TEXIFForm.InfoDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var SepPos:integer;
begin with Info,Canvas do begin
  SepPos:=pos('=',Items[Index]);
  FillRect(Rect);
  TextOut(Rect.Left+2,Rect.Top,copy(Items[Index],1,SepPos-1));
  TextOut(Rect.Left+5+Spacing,Rect.Top,copy(Items[Index],SepPos+1,255));
end; end;

end.
