unit List;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TListForm = class(TForm)
    ListBox: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListBoxClick(Sender: TObject);
    procedure ListBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  private
    { Private-Deklarationen }
    PathPos:integer;
  public
    { Public-Deklarationen }
    procedure RefreshList;
    procedure RemoveFromList;
  end;

var
  ListForm: TListForm;

implementation

uses Main;

{$R *.DFM}

procedure TListForm.FormCreate(Sender: TObject);
begin
  Font:=MainForm.Font;
  RefreshList;
end;

procedure TListForm.RefreshList;
var ImageNo:integer;
    CurWidth,MaxWidth:integer;
begin
  MaxWidth:=0;
  with ListBox,Items,MainForm do begin
    Clear;
    for ImageNo:=0 to (ImageCount-1) do
      with Images[ImageNo] do begin
        CurWidth:=Self.Canvas.TextWidth(FileName);
        if CurWidth>MaxWidth then MaxWidth:=CurWidth;
        Add(FilePath+FileName);
      end;
    ItemIndex:=MainForm.CurrentImage;
  end;
  PathPos:=MaxWidth+38;
end;

procedure TListForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    13,27:Hide;
    32:MainForm.MSelectClick(nil);
    80:MainForm.MFullPathClick(nil);
    46:RemoveFromList;
  end;
end;

procedure TListForm.ListBoxClick(Sender: TObject);
begin
  MainForm.Go(ListBox.ItemIndex);
end;

procedure TListForm.ListBoxDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var No:string[3];
begin
  with MainForm,ListBox.Canvas do begin
    FillRect(Rect);
    if Index>=ImageCount then exit;
    str(Index+1,No);
    TextOut(Rect.Left+21-TextWidth(No),Rect.Top+1,No);
    if Images[Index].Selected then
      TextOut(Rect.Left+23,Rect.Top+1,#149);
    if FullPath then
      TextOut(Rect.Left+30,Rect.Top+1,ListBox.Items[Index])
    else begin
      TextOut(Rect.Left+30,Rect.Top+1,Images[Index].FileName);
      if odSelected in State then Font.Color:=clBtnHighlight else Font.Color:=clGrayText;
      TextOut(Rect.Left+PathPos,Rect.Top+1,ListBox.Items[Index]);
    end;
  end;
end;

procedure TListForm.RemoveFromList;
begin
  with MainForm do begin
    if ImageCount<1 then exit;
    dec(ImageCount);
    if ImageCount>0 then
      Move(Images[CurrentImage+1],Images[CurrentImage],(ImageCount-CurrentImage)*sizeof(TImgRec));
    SetLength(Images,ImageCount);
    if CurrentImage>=ImageCount then CurrentImage:=ImageCount-1;
    Go(CurrentImage);
    ListBox.Items.Delete(CurrentImage);
    ListBox.ItemIndex:=CurrentImage;
  end;
end;

end.
