unit Main;

{$UNDEF USE_PROGRAM_DIRECTORY}
{$DEFINE ZOOM_BY_DEFAULT}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, JPEG,
  Menus, ShellAPI, XPMan;

const PATH_DELIM='\';
      DEFAULT_IMAGE_LIST='DEFAULT.IL';
      EMPTY_LIST='Keine Bilder verfgbar';
      NO_MORE_IMAGES='Keine weiteren Bilder.';
      LOGO_Y_ADJUST=31;

type str63=string[63];
type TImgRec=record
               FilePath:shortstring;
               FileName:str63;
               Selected:boolean;
             end;

type
  TMainForm = class(TForm)
    Popup: TPopupMenu;
    MQuit: TMenuItem;
    MNextImage: TMenuItem;
    MPrevImage: TMenuItem;
    MQuality: TMenuItem;
    N1: TMenuItem;
    MList: TMenuItem;
    MSelect: TMenuItem;
    N2: TMenuItem;
    MSelectModify: TMenuItem;
    MSelectAll: TMenuItem;
    MSelectNone: TMenuItem;
    MSelectInvert: TMenuItem;
    N3: TMenuItem;
    MSelectedOnly: TMenuItem;
    MRedraw: TMenuItem;
    MOperations: TMenuItem;
    MSetFolder: TMenuItem;
    N4: TMenuItem;
    MSelCopy: TMenuItem;
    MSelMove: TMenuItem;
    SetFolderDlg: TSaveDialog;
    MShowStatus: TMenuItem;
    MNiceText: TMenuItem;
    OpenDialog: TOpenDialog;
    MReOpen: TMenuItem;
    N5: TMenuItem;
    MFullPath: TMenuItem;
    MSearch: TMenuItem;
    MSelDelete: TMenuItem;
    MRemove: TMenuItem;
    MSelSave: TMenuItem;
    SaveListDlg: TSaveDialog;
    MZoom: TMenuItem;
    MShowEXIF: TMenuItem;
    XPManifest: TXPManifest;
    procedure FormCreate(Sender: TObject);
    procedure OnEraseBkgnd(var msg:TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MQuitClick(Sender: TObject);
    procedure MNextImageClick(Sender: TObject);
    procedure MPrevImageClick(Sender: TObject);
    procedure MQualityClick(Sender: TObject);
    procedure PopupPopup(Sender: TObject);
    procedure MListClick(Sender: TObject);
    procedure MSelectClick(Sender: TObject);
    procedure MSelectAllClick(Sender: TObject);
    procedure MSelectNoneClick(Sender: TObject);
    procedure MSelectInvertClick(Sender: TObject);
    procedure MRedrawClick(Sender: TObject);
    procedure MSelectedOnlyClick(Sender: TObject);
    procedure MSetFolderClick(Sender: TObject);
    procedure MSelCopyClick(Sender: TObject);
    procedure MSelMoveClick(Sender: TObject);
    procedure MShowStatusClick(Sender: TObject);
    procedure MNiceTextClick(Sender: TObject);
    procedure MReOpenClick(Sender: TObject);
    procedure MFullPathClick(Sender: TObject);
    procedure MSearchClick(Sender: TObject);
    procedure MSelDeleteClick(Sender: TObject);
    procedure MRemoveClick(Sender: TObject);
    procedure MSelSaveClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormDropFiles(var msg:TMessage); message WM_DROPFILES;
    procedure MZoomClick(Sender: TObject);
    procedure MShowEXIFClick(Sender: TObject);
  private
    { Private-Deklarationen }
    StatusPos:integer;
    procedure LoadDirectory(const BaseDir:string);
    procedure LoadList(const BaseDir,ListName:string);
    procedure MassSelection(Norm,Inv:boolean);
    procedure CopyMove(MoveMode:boolean);
    function ConfirmSelOperation(const OpType:string):boolean;
    procedure Remove(Commit:boolean);
  public
    { Public-Deklarationen }
    MaxX,MaxY:integer;
    BMP:TBitmap;
    Logo:TBitmap;
    LogoX,LogoY:integer;

    Zoom:boolean;
    ShowStatus:boolean;
    NiceText:boolean;
    FullPath:boolean;
    SelectedOnly:boolean;
    HighQuality:boolean;
    Status:string;

    DestDir:string;
    ImageCountStr:string[7];
    CurrentSize:string[15];
    CurrentImage:integer;
    CurrentImagePath:string;
    Images:array of TImgRec;
    ImageCount:integer;

    procedure RefreshList(const BasePath:string);
    procedure Go(ImageNo:integer);
    procedure LoadImage(const FileName:string);
    procedure DrawText(X,Y:integer;const Text:string);
    procedure SortList(const Unsorted:array of TImgRec);
  end;

var
  MainForm: TMainForm;

implementation

uses List, Overwrite, Search, EXIFView;

{$R *.DFM}
{$R TVLOGO.RES}

procedure TMainForm.LoadDirectory(const BaseDir:string);
var SR:TSearchRec; ContinueFind:integer;
    Unsorted:array of TImgRec;
    ImageNo:integer;
begin
  ImageCount:=0;
  ContinueFind:=FindFirst(BaseDir+'*.jpg',$27,SR);
  while ContinueFind=0 do begin
    inc(ImageCount);
    ContinueFind:=FindNext(SR);
  end;
  FindClose(SR);
  if (ImageCount<=0) then begin
    SetLength(Images,0);
  exit;end;

  SetLength(Unsorted,ImageCount);
  ImageNo:=0;
  ContinueFind:=FindFirst(BaseDir+'*.jpg',$27,SR);
  while ContinueFind=0 do begin
    with Unsorted[ImageNo] do begin
      FilePath:=BaseDir;
      FileName:=SR.Name;
      Selected:=false;
    end;
    inc(ImageNo);
    ContinueFind:=FindNext(SR);
  end;
  FindClose(SR);

  SortList(Unsorted);
end;

procedure TMainForm.SortList(const Unsorted:array of TImgRec);
var Upcased:array of str63;
    Order:array of integer;
    ImageNo:integer;
  procedure qsort(lo,hi:integer);
  var a,b,pivot:integer;
  begin
    a:=lo; b:=hi;
    pivot:=Order[(a+b)SHR 1];
    repeat
      while Upcased[Order[a]]<Upcased[pivot] do inc(a);
      while Upcased[Order[b]]>Upcased[pivot] do dec(b);
      if a<=b then begin
        ImageNo:=Order[a];
        Order[a]:=Order[b];
        Order[b]:=ImageNo;
        inc(a);
        dec(b);
      end;
    until a>b;
    if b>lo then qsort(lo,b);
    if a<hi then qsort(a,hi);
  end;
begin
  SetLength(Order,ImageCount);
  for ImageNo:=0 to (ImageCount-1) do
    Order[ImageNo]:=ImageNo;

  SetLength(Upcased,ImageCount);
  for ImageNo:=0 to (ImageCount-1) do
    Upcased[ImageNo]:=UpperCase(Unsorted[ImageNo].FileName);
  qsort(0,ImageCount-1);
  SetLength(Upcased,0);

  SetLength(Images,ImageCount);
  for ImageNo:=0 to (ImageCount-1) do
    Images[ImageNo]:=Unsorted[Order[ImageNo]];
end;

procedure TMainForm.LoadList(const BaseDir,ListName:string);
var ListFile:System.Text;
    Line:string;
    ImageNo:integer;
    LastPathDelim:integer;
  function Validate:boolean;
  begin
    Validate:=false;
    LastPathDelim:=pos(^I'#',Line);
    if LastPathDelim>0 then SetLength(Line,LastPathDelim-1);
    Line:=Trim(Line);
    if length(Line)<5 then exit;
    if UpperCase(copy(Line,length(Line)-3,4))<>'.JPG' then exit;
    if not((Line[1] IN ['A'..'Z']) AND (Line[2]=':') AND (Line[3]='\')) then begin
      LastPathDelim:=length(BaseDir);
      while copy(Line,1,3)='..\' do begin
        Delete(Line,1,3);
        dec(LastPathDelim);
        while (LastPathDelim>0) AND (BaseDir[LastPathDelim]<>PATH_DELIM) do dec(LastPathDelim);
      end;
      Line:=copy(BaseDir,1,LastPathDelim)+Line;
    end;
    if not FileExists(Line) then exit;
    Validate:=true;
  end;
begin
  AssignFile(ListFile,BaseDir+ListName);
  {$I-} Reset(ListFile); {$I+}
  if IOResult<>0 then begin
    Status:='Konnte Bilderliste nicht ffnen!';
    exit;
  end;

  ImageCount:=0;
  while not EOF(ListFile) do begin
    readln(ListFile,Line);
    if Validate then inc(ImageCount);
  end;
  CloseFile(ListFile);

  SetLength(Images,ImageCount);
  if ImageCount<=0 then exit;

  ImageNo:=0;
  Reset(ListFile);
  while not EOF(ListFile) do begin
    readln(ListFile,Line);
    if Validate then begin
      LastPathDelim:=length(Line);
      while (Line[LastPathDelim]<>PATH_DELIM) do dec(LastPathDelim);
      with Images[ImageNo] do begin
        FilePath:=copy(Line,1,LastPathDelim);
        FileName:=copy(Line,LastPathDelim+1,63);
        Selected:=false;
      end;
      inc(ImageNo);
    end;
  end;
  CloseFile(ListFile);
end;

procedure TMainForm.RefreshList(const BasePath:string);
var BaseDir,FirstFile:string; FileStatus:integer; LastPathDelim:integer;
    temp:string; ImageNo:integer;
begin
  BaseDir:=BasePath; FirstFile:=''; CurrentImage:=0; 
  if BaseDir='' then begin
{$IFDEF USE_PROGRAM_DIRECTORY}
    BaseDir:=ExtractFilePath(ExpandUNCFileName(ParamStr(0)));
    ChDir(BaseDir);
{$ELSE}
    GetDir(0,BaseDir);
{$ENDIF}
    if BaseDir[length(BaseDir)]<>PATH_DELIM then BaseDir:=BaseDir+PATH_DELIM;
    if FileExists(DEFAULT_IMAGE_LIST) then FirstFile:=DEFAULT_IMAGE_LIST;
  end else begin
    temp:=BaseDir+#0;
    FileStatus:=GetFileAttributes(@temp[1]);
    if FileStatus=-1 then begin
      GetDir(0,BaseDir);
      if BaseDir[length(BaseDir)]<>PATH_DELIM then BaseDir:=BaseDir+PATH_DELIM;
    end else if ((FileStatus AND FILE_ATTRIBUTE_DIRECTORY)<>0) then begin
      if BaseDir[length(BaseDir)]<>PATH_DELIM then BaseDir:=BaseDir+PATH_DELIM;
    end else begin
      LastPathDelim:=length(BaseDir);
      while (LastPathDelim>0) AND (BaseDir[LastPathDelim]<>PATH_DELIM) do dec(LastPathDelim);
      FirstFile:=UpperCase(copy(BaseDir,LastPathDelim+1,63));
      SetLength(BaseDir,LastPathDelim);
    end;
  end;

  if (length(FirstFile)>3) AND (copy(FirstFile,length(FirstFile)-2,3)='.IL') then
    LoadList(BaseDir,FirstFile)
  else begin
    LoadDirectory(BaseDir);
    for ImageNo:=0 to (ImageCount-1) do
      if FirstFile=UpperCase(Images[ImageNo].FileName) then
        CurrentImage:=ImageNo;
  end;

  ImageCountStr:='/'+IntToStr(ImageCount)+'] ';
  if Assigned(ListForm) then ListForm.RefreshList;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var Metrics:TNonClientMetrics;
begin
AllocConsole;
  randomize;
  DecimalSeparator:='.';
  
  MaxX:=Screen.Width;
  MaxY:=Screen.Height;
  Top:=0; Left:=0;
  Width:=MaxX; Height:=MaxY;
  BMP:=TBitmap.Create;
  with BMP do begin
    PixelFormat:=pfDevice;
    Width:=MaxX;
    Height:=MaxY;
    with Canvas do begin
      Pen.Style:=psClear;
      Brush.Style:=bsSolid;
      Brush.Color:=0;
    end;
  end;
  Logo:=TBitmap.Create;
  Logo.LoadFromResourceName(MainInstance,'TVLOGO');
  LogoX:=(MaxX-Logo.Width)SHR 1;
  LogoY:=(MaxY SHR 1)-LOGO_Y_ADJUST;

  Metrics.cbSize:=sizeof(Metrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS,sizeof(Metrics),@Metrics,0) then
    with Metrics.lfMenuFont do begin
      Font.Height:=lfHeight;
      Font.Name:=lfFaceName;
    end;
  StatusPos:=MaxY-5-Canvas.TextHeight('X');

{$IFDEF USE_PROGRAM_DIRECTORY}
  DestDir:=ExtractFilePath(ExpandUNCFileName(ParamStr(0)));
{$ELSE}
  GetDir(0,DestDir);
{$ENDIF}
  if DestDir[length(DestDir)]<>PATH_DELIM then DestDir:=DestDir+PATH_DELIM;
  RefreshList(ParamStr(1));

{$IFDEF ZOOM_BY_DEFAULT}
  Zoom:=true;
{$ELSE}
  Zoom:=false;
{$ENDIF}
  ShowStatus:=true;
  NiceText:=true;
  FullPath:=false;
  SelectedOnly:=false;
  HighQuality:=false;

  Status:='';
  Go(CurrentImage);
end;

procedure TMainForm.OnEraseBkgnd(var msg:TWMEraseBkgnd);
begin
  msg.Result:=0;
end;

procedure TMainForm.LoadImage(const FileName:string);
var JPG:TJPEGImage;
    SizeX,SizeY:integer;
    NewX,NewY:integer;
    BaseX,BaseY:integer;
  procedure NoImage;
  begin
    BMP.Canvas.FillRect(Rect(0,0,MaxX,MaxY));
    BMP.Canvas.Draw(LogoX,LogoY,Logo);
    CurrentSize:='';
  end;
begin
  CurrentImagePath:=FileName;
  if Assigned(EXIFForm) then EXIFForm.ReadImage();
  if FileName='' then begin NoImage; exit; end;
  JPG:=TJPEGImage.Create;
  with JPG do begin
    ProgressiveDisplay:=false;
    if not HighQuality then Performance:=jpBestSpeed;
    try
      LoadFromFile(FileName);
      SizeX:=Width; SizeY:=Height;
    except
      NoImage;
      JPG.Free;
      exit;
    end;
  end;
  CurrentSize:=' ('+IntToStr(SizeX)+'x'+IntToStr(SizeY)+')';
  if Zoom then NewX:=MaxX
          else if SizeX>MaxX then NewX:=MaxX else NewX:=SizeX;
  NewY:=SizeY*NewX DIV SizeX;
  if NewY>MaxY then begin
    NewX:=SizeX*MaxY DIV SizeY;
    NewY:=MaxY;
  end;
  BaseX:=(MaxX-NewX)SHR 1;
  BaseY:=(MaxY-NewY)SHR 1;
  with BMP.Canvas do begin
    FillRect(Rect(0,0,MaxX,MaxY));
    try
      StretchDraw(Rect(BaseX,BaseY,BaseX+NewX,BaseY+NewY),JPG);
    except
      NoImage;
    end;
  end;
  JPG.Free;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Logo.Free;
  BMP.Free;
end;

procedure TMainForm.FormPaint(Sender: TObject);
var s:shortstring;
begin
  Canvas.Draw(0,0,BMP);
  if ShowStatus then begin
    if ImageCount=0 then
      DrawText(5,3,EMPTY_LIST)
    else begin
      s:='['+IntToStr(CurrentImage+1)+ImageCountStr;
      with Images[CurrentImage] do begin
        if Selected then s:=s+#149#32;
        if FullPath then s:=s+FilePath;
        DrawText(5,3,s+FileName+CurrentSize);
      end;
    end;
  end;
  if length(Status)>0 then begin
    DrawText(5,StatusPos,Status);
    Status:='';
  end;
end;

procedure TMainForm.DrawText(X,Y:integer;const Text:string);
begin
  with Canvas do begin
    if NiceText then begin
      Brush.Style:=bsClear;
      Font.Color:=0;
      TextOut(X+1,Y,Text);
      TextOut(X,Y+1,Text);
      TextOut(X+1,Y+1,Text);
    end else begin
      Brush.Style:=bsSolid;
      Brush.Color:=0;
    end;
    Font.Color:=$FFFFFF;
    TextOut(X,Y,Text);
  end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Width:=116; Height:=27;
  BorderStyle:=bsSingle;
end;

procedure TMainForm.MQuitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.Go(ImageNo:integer);
begin
  if ImageCount=0 then begin
    LoadImage('');
    Repaint;
  exit; end;
  if ImageNo<0 then ImageNo:=0;
  if ImageNo>=ImageCount then ImageNo:=ImageCount-1;
  CurrentImage:=ImageNo;
  with Images[CurrentImage] do
    LoadImage(FilePath+FileName);
  if Assigned(ListForm) then ListForm.ListBox.ItemIndex:=CurrentImage;
  Repaint;
end;

procedure TMainForm.MNextImageClick(Sender: TObject);
var ImageNo:integer;
begin
  if SelectedOnly then begin
    ImageNo:=CurrentImage;
    repeat
      inc(ImageNo);
    until (ImageNo>=ImageCount) OR (Images[ImageNo].Selected);
    if (ImageNo>=ImageCount) then begin
      Status:=NO_MORE_IMAGES;
      Invalidate;
    end else
      Go(ImageNo);
  end else
    if (CurrentImage+2>ImageCount) then begin
      Status:=NO_MORE_IMAGES;
      Invalidate;
    end else
      Go(CurrentImage+1);
end;

procedure TMainForm.MPrevImageClick(Sender: TObject);
var ImageNo:integer;
begin
  if SelectedOnly then begin
    ImageNo:=CurrentImage;
    repeat
      dec(ImageNo);
    until (ImageNo<0) OR (Images[ImageNo].Selected);
    if (ImageNo<0) then begin
      Status:=NO_MORE_IMAGES;
      Invalidate;
    end else
      Go(ImageNo);
  end else
    if (CurrentImage<=0) then begin
      Status:=NO_MORE_IMAGES;
      Invalidate;
    end else
      Go(CurrentImage-1);
end;

procedure TMainForm.MQualityClick(Sender: TObject);
begin
  HighQuality:=not HighQuality;
  if HighQuality
    then Status:='Bilddecoder: Hohe Qualitt'
    else Status:='Bilddecoder: Hohe Geschwindigkeit';
  Go(CurrentImage);
end;

procedure TMainForm.PopupPopup(Sender: TObject);
begin
  MList.Checked:=ListForm.Visible;
  MShowEXIF.Checked:=EXIFForm.Visible;
  if ImageCount>0 then begin
    MSelect.Enabled:=true;
    MSelect.Checked:=Images[CurrentImage].Selected;
  end else begin
    MSelect.Enabled:=false;
    MSelect.Checked:=false;
  end;
  MZoom.Checked:=Zoom;
  MShowStatus.Checked:=ShowStatus;
  MNiceText.Checked:=NiceText;
  MFullPath.Checked:=FullPath;
  MSelectedOnly.Checked:=SelectedOnly;
  MQuality.Checked:=HighQuality;
end;

procedure TMainForm.MListClick(Sender: TObject);
begin
  if ListForm.Visible then ListForm.Hide else ListForm.Show;
end;

procedure TMainForm.MSelectClick(Sender: TObject);
begin
  if ImageCount<0 then exit;
  Images[CurrentImage].Selected:=not(Images[CurrentImage].Selected);
  Invalidate;
  ListForm.ListBox.Invalidate;
end;

procedure TMainForm.MSelectAllClick(Sender: TObject);
begin
  MassSelection(true,true);
end;

procedure TMainForm.MSelectNoneClick(Sender: TObject);
begin
  MassSelection(false,false);
end;

procedure TMainForm.MSelectInvertClick(Sender: TObject);
begin
  MassSelection(false,true);
end;

procedure TMainForm.MassSelection(Norm,Inv:boolean);
var ImageNo:integer; State:boolean;
begin
  for ImageNo:=0 to (ImageCount-1) do begin
    State:=Images[ImageNo].Selected;
    Images[ImageNo].Selected:=(Norm AND State) OR (Inv AND not(State));
  end;
  Invalidate;
  ListForm.ListBox.Invalidate;
end;

procedure TMainForm.MRedrawClick(Sender: TObject);
begin
  Repaint;
end;

procedure TMainForm.MSelectedOnlyClick(Sender: TObject);
begin
  SelectedOnly:=not SelectedOnly;
end;

procedure TMainForm.MSetFolderClick(Sender: TObject);
var LastPathDelim:integer;
begin
  SetFolderDlg.InitialDir:=DestDir;
  SetFolderDlg.FileName:='Dateiname ist egal';
  if SetFolderDlg.Execute then begin
    DestDir:=SetFolderDlg.FileName;
    LastPathDelim:=length(DestDir);
    while DestDir[LastPathDelim]<>PATH_DELIM do dec(LastPathDelim);
    SetLength(DestDir,LastPathDelim);
    Status:='Zielordner auf '#187+DestDir+#171' gendert';    
    Invalidate;
  end;
end;

procedure TMainForm.MSelCopyClick(Sender: TObject);
begin
  if ConfirmSelOperation('kopiert') then CopyMove(false);
end;

procedure TMainForm.MSelMoveClick(Sender: TObject);
begin
  if ConfirmSelOperation('verschoben') then CopyMove(true);
end;

procedure TMainForm.CopyMove(MoveMode:boolean);
var ImageNo,FileCount:integer;
    SourceName,DestName:string;
    DoIt:boolean; Result:TModalResult;
begin
  FileCount:=0;
  for ImageNo:=0 to (ImageCount-1) do with Images[ImageNo] do if Selected then begin
    SourceName:=FilePath+FileName+#0;
    DestName:=DestDir+FileName+#0;
    if FileExists(DestName) then begin
      Go(ImageNo);
      OverwriteForm.FilePath:=Images[ImageNo].FilePath;
      OverwriteForm.FileName:=Images[ImageNo].FileName;
      Result:=OverwriteForm.ShowModal;
      DoIt:=(Result=mrYes);
      if DoIt then
        DestName:=DestDir+OverwriteForm.FileName+#0;
      if (Result=mrCancel) then break;
    end else
      DoIt:=true;
    if DoIt then begin
      if MoveMode then begin
        DeleteFile(DestName);
        if not MoveFile(@SourceName[1],@DestName[1]) then begin
          CopyFile(@SourceName[1],@DestName[1],false);
          DeleteFile(SourceName);
        end;
      end else
        CopyFile(@SourceName[1],@DestName[1],false);
      inc(FileCount);
    end;
  end;
  if MoveMode
    then Status:=IntToStr(FileCount)+' Dateien verschoben'
    else Status:=IntToStr(FileCount)+' Dateien kopiert';
  Invalidate;
end;

procedure TMainForm.MShowStatusClick(Sender: TObject);
begin
  ShowStatus:=not ShowStatus;
  Invalidate;
end;

procedure TMainForm.MNiceTextClick(Sender: TObject);
begin
  NiceText:=not NiceText;
  Invalidate;
end;

procedure TMainForm.MReOpenClick(Sender: TObject);
begin
  with OpenDialog do begin
    if ImageCount>0 then begin
      FileName:=Images[CurrentImage].FilePath+Images[CurrentImage].FileName;
      InitialDir:=Images[CurrentImage].FilePath;
    end else begin
      FileName:='';
      InitialDir:='.';
    end;
    if Execute then begin
      RefreshList(FileName);
      Go(CurrentImage);
    end;
  end;
end;

procedure TMainForm.MFullPathClick(Sender: TObject);
begin
  FullPath:=not(FullPath);
  Invalidate;
  ListForm.ListBox.Invalidate;
end;

procedure TMainForm.MSearchClick(Sender: TObject);
begin
  SearchForm.ShowModal;
end;

procedure TMainForm.MSelDeleteClick(Sender: TObject);
begin
  if ConfirmSelOperation('gelscht')
    then Remove(true);
end;

function TMainForm.ConfirmSelOperation(const OpType:string):boolean;
var SelCount,LastSel:integer;
    ImageNo:integer;
begin
  SelCount:=0; LastSel:=CurrentImage;
  for ImageNo:=0 to (ImageCount-1) do
    if Images[ImageNo].Selected then begin
      inc(SelCount);
      LastSel:=ImageNo;
    end;
  if SelCount<2 then begin
    ConfirmSelOperation:=
      (MessageDlg('Soll die Datei'^M^J#32#32+
                  Images[LastSel].FilePath+Images[LastSel].FileName+
                  ^M^J+OpType+' werden?',
                  mtConfirmation,[mbYes,mbNo],0)=mrYes);
    if Result then Images[LastSel].Selected:=true;
  end else
    ConfirmSelOperation:=
      (MessageDlg('Sollen die '+IntToStr(SelCount)+' ausgewhlten Dateien '+
                  OpType+' werden?',mtConfirmation,[mbYes,mbNo],0)=mrYes);
end;

procedure TMainForm.MRemoveClick(Sender: TObject);
begin
  if ConfirmSelOperation('aus der Liste gelscht')
    then Remove(false);
end;

procedure TMainForm.Remove(Commit:boolean);
var ImageNo,DelCount:integer;
begin
  ImageNo:=0; DelCount:=0;
  while ImageNo<ImageCount do
    with Images[ImageNo] do
      if Selected then begin
        if Commit AND DeleteFile(FilePath+FileName) then inc(DelCount);
        dec(ImageCount);
        Move(Images[ImageNo+1],Images[ImageNo],(ImageCount-ImageNo)*sizeof(TImgRec));
        if CurrentImage>ImageNo then dec(CurrentImage);
      end else
        inc(ImageNo);
  SetLength(Images,ImageCount);
  ImageCountStr:='/'+IntToStr(ImageCount)+'] ';
  ListForm.RefreshList;
  if Commit then Status:=IntToStr(DelCount)+' Dateien gelscht';
  Go(CurrentImage);
end;

procedure TMainForm.MSelSaveClick(Sender: TObject);
var SelCount,ImageNo:integer;
    PathC:string; LastPathDelim:integer;
    RelativePath:boolean;
    ListFile:System.Text;
begin
  SelCount:=0;
  for ImageNo:=0 to (ImageCount-1) do
    if Images[ImageNo].Selected then inc(SelCount);
  if SelCount=0 then begin
    MessageDlg('Es sind gar keine Bilder ausgewhlt!',mtError,[mbOK],0);
    exit;
  end;
  SaveListDlg.InitialDir:=Images[CurrentImage].FilePath;
  if not SaveListDlg.Execute then exit;

  PathC:=SaveListDlg.FileName;
  LastPathDelim:=length(PathC);
  while (PathC[LastPathDelim]<>PATH_DELIM) do dec(LastPathDelim);
  PathC:=UpperCase(copy(PathC,1,LastPathDelim));
  RelativePath:=true;
  for ImageNo:=0 to (ImageCount-1) do
    with Images[ImageNo] do
      if Selected then
        if copy(UpperCase(FilePath),1,length(PathC))<>PathC then begin
          RelativePath:=false;
          break;
        end;

  AssignFile(ListFile,SaveListDlg.FileName);
  {$I-} Rewrite(ListFile); {$I+}
  if IOResult<>0 then begin
    MessageDlg('Fehler beim Schreiben der Listendatei!',mtError,[mbOK],0);
    exit;
  end;
  for ImageNo:=0 to (ImageCount-1) do
    with Images[ImageNo] do
      if Selected then begin
        if RelativePath
          then writeln(ListFile,copy(FilePath,LastPathDelim+1,260),FileName)
          else writeln(ListFile,FilePath,FileName);
      end;
  CloseFile(ListFile);
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  DragAcceptFiles(Handle,true);
end;

procedure TMainForm.FormHide(Sender: TObject);
begin
  DragAcceptFiles(Handle,false);
end;

procedure TMainForm.FormDropFiles(var msg:TMessage);
var hDrop:THandle;
    Buf:array[0..MAX_PATH]of char;
begin
  hDrop:=msg.wParam;
  msg.Result:=0;
  Buf[0]:=#0;
  if integer(DragQueryFile(hDrop,cardinal(-1),nil,0))>0 then
    DragQueryFile(hDrop,0,@Buf,MAX_PATH);
  DragFinish(hDrop);
  if Buf[0]<>#0 then begin
    RefreshList(Buf);
    Go(CurrentImage);
  end;
end;

procedure TMainForm.MZoomClick(Sender: TObject);
begin
  Zoom:=not Zoom;
  Go(CurrentImage);
end;

procedure TMainForm.MShowEXIFClick(Sender: TObject);
begin
  EXIFForm.Visible:=not(EXIFForm.Visible);
end;

end.
