unit Overwrite;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

const STANDARD_WIDTH=285;
      EXTENDED_WIDTH=501;
      MEDIUM_WIDTH=(EXTENDED_WIDTH+STANDARD_WIDTH)SHR 1;

const G_NUMBERS='0123456789';
      G_HEX='0123456789abcdef';
      G_LETTERS='abcdefghijklmnopqrstuvwxyz';
      G_ALL=G_NUMBERS+G_LETTERS;

type
  TOverwriteForm = class(TForm)
    MOverwrite: TRadioButton;
    MCreateSuffix: TRadioButton;
    MRename: TRadioButton;
    MIgnore: TRadioButton;
    ENewName: TEdit;
    Label1: TLabel;
    LDestName: TLabel;
    Label2: TLabel;
    ActionBevel: TBevel;
    BExtend: TButton;
    BOK: TButton;
    LSourceName: TLabel;
    LDestInfo: TLabel;
    LSourceInfo: TLabel;
    Label3: TLabel;
    DestBevel1: TBevel;
    SourceBevel1: TBevel;
    DestBevel2: TBevel;
    SourceBevel2: TBevel;
    SettingsBox: TGroupBox;
    LSuffixStart: TLabel;
    ESuffixStart: TEdit;
    ESuffixLength: TEdit;
    ESuffixEnd: TEdit;
    LSuffixLength: TLabel;
    LSuffixEnd: TLabel;
    CSuffixLength: TUpDown;
    LCharacterType: TLabel;
    GNumbers: TRadioButton;
    GHex: TRadioButton;
    GLetters: TRadioButton;
    GAll: TRadioButton;
    LNumbers: TLabel;
    LHex: TLabel;
    LLetters: TLabel;
    LAll: TLabel;
    BGenerate: TButton;
    GCapitals: TCheckBox;
    BShowDest: TButton;
    procedure FormCreate(Sender: TObject);
    procedure BOKClick(Sender: TObject);
    procedure ModeChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BExtendClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure BGenerateClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure BShowDestClick(Sender: TObject);
  private
    { Private-Deklarationen }
    LastDot:integer;
  public
    { Public-Deklarationen }
    FilePath:string;
    FileName:string;
    procedure GenerateName;
  end;

var
  OverwriteForm: TOverwriteForm;

implementation

uses Main, Preview;

{$R *.DFM}

function GetFileInfoStr(const FileName:string):string;
var temp:string; tpos:integer;
    hFile:THandle; FileInfo:TByHandleFileInformation;
begin
  temp:=FileName+#0;
  hFile:=CreateFile(@temp[1],0,0,nil,OPEN_EXISTING,0,0);
  if (hFile=INVALID_HANDLE_VALUE) OR not(GetFileInformationByHandle(hFile,FileInfo)) then begin
    GetFileInfoStr:='[Datei-Informationen knnen nicht abgerufen werden]';
    exit;
  end;
  CloseHandle(hFile);

  temp:=IntToStr(FileInfo.nFileSizeLow);
  tpos:=length(temp)-2;
  while tpos>1 do begin
    Insert('.',temp,tpos);
    dec(tpos,3);
  end;

  GetFileInfoStr:='('+temp+' Bytes)';
end;

procedure TOverwriteForm.FormCreate(Sender: TObject);
begin
  Font:=MainForm.Font;
  LDestName.Font.Style:=[fsBold];
  LSourceName.Font.Style:=[fsBold];
  LSuffixStart.ParentFont:=false;
  ESuffixStart.ParentFont:=false;
  LSuffixEnd.ParentFont:=false;
  ESuffixEnd.ParentFont:=false;
  LSuffixLength.ParentFont:=false;
  ESuffixLength.ParentFont:=false;
  GNumbers.ParentFont:=false;
  LNumbers.ParentFont:=false;
  GHex.ParentFont:=false;
  LHex.ParentFont:=false;
  GLetters.ParentFont:=false;
  LLetters.ParentFont:=false;
  GAll.ParentFont:=false;
  LAll.ParentFont:=false;
  GCapitals.ParentFont:=false;
  BGenerate.ParentFont:=false;
  SettingsBox.Font.Style:=[fsBold];

  Width:=STANDARD_WIDTH;
end;

procedure TOverwriteForm.BOKClick(Sender: TObject);
begin
  if MIgnore.Checked then ModalResult:=mrNo else ModalResult:=mrYes;
end;

procedure TOverwriteForm.ModeChange(Sender: TObject);
begin
  ENewName.Enabled:=MRename.Checked;
  if MOverwrite.Checked then ENewName.Text:=FileName;
  GenerateName;
end;

procedure TOverwriteForm.FormShow(Sender: TObject);
var s:string;
begin
  LastDot:=length(FileName); while (FileName[LastDot]<>'.') do dec(LastDot);
  Caption:=FileName;
  ENewName.Text:=FileName;
  ModeChange(nil);
  s:=MainForm.DestDir+FileName;
  LDestName.Caption:=s;
  LDestInfo.Caption:=GetFileInfoStr(s);
  s:=FilePath+FileName;
  LSourceName.Caption:=s;
  LSourceInfo.Caption:=GetFileInfoStr(s);
end;

procedure TOverwriteForm.GenerateName;
var suf:string[23]; sufpos:byte;
    gamut:string[39]; ch:char;
    upper:boolean;
begin
  if not MCreateSuffix.Checked then exit;
       if GNumbers.Checked then gamut:=G_NUMBERS
  else if GHex    .Checked then gamut:=G_HEX
  else if GLetters.Checked then gamut:=G_LETTERS
  else                          gamut:=G_ALL;
  upper:=GCapitals.Checked;
  suf[0]:=char(CSuffixLength.Position);
  for sufpos:=1 to length(suf) do begin
    ch:=gamut[random(length(gamut))+1];
    if upper then ch:=upcase(ch);
    suf[sufpos]:=ch;
  end;
  ENewName.Text:=copy(FileName,1,LastDot-1)+ESuffixStart.Text+suf+
                 ESuffixEnd.Text+copy(FileName,LastDot,5);
end;

procedure TOverwriteForm.BExtendClick(Sender: TObject);
begin
  if Width<MEDIUM_WIDTH then begin
    Width:=EXTENDED_WIDTH;
    BExtend.Caption:='Standard &<<';
  end else begin
    Width:=STANDARD_WIDTH;
    BExtend.Caption:='Erweitert &>>';
  end;
end;

procedure TOverwriteForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#27 then ModalResult:=mrCancel;
end;

procedure TOverwriteForm.BGenerateClick(Sender: TObject);
begin
  GenerateName;
end;

procedure TOverwriteForm.FormHide(Sender: TObject);
begin
  FileName:=ENewName.Text;
end;

procedure TOverwriteForm.BShowDestClick(Sender: TObject);
begin
  PreviewForm.FileName:=LDestName.Caption;
  PreviewForm.ShowModal;
end;

end.
