unit SThread;

interface

uses
  Classes, SysUtils, FileCtrl;

type
  TSearchThread = class(TThread)
  private
    { Private-Deklarationen }
    TempStr:string;
    FoundCount:integer;
    FolderCount:integer;
    function HandlePath(const Path:string):boolean;
    procedure SetStatus;
    procedure PutFile;
    procedure EndSuccess;
    procedure EndFail;
  public
    BaseDir,Pattern:string;
    AtStart,SubDirs:boolean;
  protected
    procedure Execute; override;
  end;

implementation

uses Search;

procedure TSearchThread.SetStatus;
begin
  SearchForm.Info.Caption:=TempStr;
end;

procedure TSearchThread.PutFile;
begin
  SearchForm.FileList.Items.Add(TempStr);
end;

function TSearchThread.HandlePath(const Path:string):boolean;
var SR:TSearchRec; Cont:integer;
begin
  inc(FolderCount);
  TempStr:=Path; Synchronize(SetStatus);

  Cont:=FindFirst(Path+'\*.jpg',$23,SR);
  while Cont=0 do begin
    if (    AtStart  AND (copy(UpperCase(SR.Name),1,length(Pattern))=Pattern)) OR
       (not(AtStart) AND (pos(Pattern,UpperCase(SR.Name))<>0)) then
    begin
      inc(FoundCount);
      TempStr:=Path+'\'+SR.Name; Synchronize(PutFile);
    end;
    if Terminated then begin
      FindClose(SR);
      HandlePath:=false;
      exit;
    end;
    Cont:=FindNext(SR);
  end;
  FindClose(SR);

  if SubDirs then begin
    Cont:=FindFirst(Path+'\*.*',$33,SR);
    while Cont=0 do begin
      if (SR.Attr AND faDirectory<>0) AND (SR.Name<>'.') AND (SR.Name<>'..') then
          HandlePath(Path+'\'+SR.Name);
      if Terminated then begin
        FindClose(SR);
        HandlePath:=false;
        exit;
      end;
      Cont:=FindNext(SR);
    end;
    FindClose(SR);
  end;

  HandlePath:=true;
end;

procedure TSearchThread.Execute;
begin
  if (length(BaseDir)>2) AND (BaseDir[length(BaseDir)]='\')
    then SetLength(BaseDir,length(BaseDir)-1);
  FoundCount:=0; FolderCount:=0;
  if DirectoryExists(BaseDir) AND HandlePath(BaseDir)
    then Synchronize(EndSuccess)
    else Synchronize(EndFail);
end;

procedure TSearchThread.EndSuccess;
begin
  with SearchForm do begin
    Info.Caption:=IntToStr(FolderCount)+' Ordner durchsucht; '+
                  IntToStr(FoundCount)+' Datei(en) gefunden';
    BSearch.Caption:='Neu &suchen';
    BAccept.Enabled:=true;
    BCancel.Enabled:=true;
    Searching:=false;
  end;
end;

procedure TSearchThread.EndFail;
begin
  with SearchForm do begin
    Info.Caption:='Suchvorgang abgebrochen';
    BSearch.Caption:='&Suchen';
    BAccept.Enabled:=true;
    BCancel.Enabled:=true;
    Searching:=false;
  end;
end;

end.
