unit Search;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, SThread, Main;

type
  TSearchForm = class(TForm)
    LName: TLabel;
    SetFolderDlg: TSaveDialog;
    BSearch: TButton;
    BAccept: TButton;
    BCancel: TButton;
    EName: TEdit;
    CPFirst: TRadioButton;
    CPAny: TRadioButton;
    LPath: TLabel;
    EPath: TEdit;
    BBrowse: TButton;
    CSubFolders: TCheckBox;
    Info: TPanel;
    FileList: TListBox;
    CSort: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BBrowseClick(Sender: TObject);
    procedure BSearchClick(Sender: TObject);
    procedure BAcceptClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private-Deklarationen }
    procedure GetList(var List:array of TImgRec);
  public
    { Public-Deklarationen }
    Searching:boolean;
    SearchThread:TSearchThread;
  end;

var
  SearchForm: TSearchForm;

implementation

uses List;

{$R *.DFM}

procedure TSearchForm.FormCreate(Sender: TObject);
begin
  Font:=MainForm.Font;
  LName.Font.Style:=[fsBold];
  LPath.Font.Style:=[fsBold];
end;

procedure TSearchForm.FormResize(Sender: TObject);
var CWidth:integer;
begin
  CWidth:=(ClientWidth-84)SHR 1;
  CPFirst.Width:=CWidth;
  CPAny.Width:=CWidth;
  CPAny.Left:=EName.Width-CWidth;
end;

procedure TSearchForm.FormShow(Sender: TObject);
var s:string;
begin
  if EPath.Text='' then begin
    with MainForm do
      if ImageCount>0
        then s:=Images[CurrentImage].FilePath
        else s:=DestDir;
    if length(s)>3
      then EPath.Text:=copy(s,1,length(s)-1)
      else EPath.Text:=s;
  end;
end;

procedure TSearchForm.BBrowseClick(Sender: TObject);
var LastPathDelim:integer;
    TempDir:string;
begin
  SetFolderDlg.InitialDir:=EPath.Text;
  SetFolderDlg.FileName:='Dateiname ist egal';
  if not SetFolderDlg.Execute then exit;
  TempDir:=SetFolderDlg.FileName;
  LastPathDelim:=length(TempDir);
  while TempDir[LastPathDelim]<>PATH_DELIM do dec(LastPathDelim);
  if LastPathDelim>3 then dec(LastPathDelim);
  SetLength(TempDir,LastPathDelim);
  EPath.Text:=TempDir;
end;

procedure TSearchForm.BSearchClick(Sender: TObject);
begin
  if not Searching then begin
    BSearch.Caption:='&Stop';
    BAccept.Enabled:=false;
    BCancel.Enabled:=false;
    FileList.Items.Clear;
    SearchThread:=TSearchThread.Create(true);
    with SearchThread do begin
      BaseDir:=EPath.Text;
      Pattern:=UpperCase(EName.Text);
      AtStart:=CPFirst.Checked;
      SubDirs:=CSubFolders.Checked;
      FreeOnTerminate:=true;
      Searching:=true;
      Resume;
    end;
  end else
    SearchThread.Terminate;
end;

procedure TSearchForm.GetList(var List:array of TImgRec);
var Count,ImageNo:integer;
    s:string; LastPathDelim:integer;
begin
  Count:=FileList.Items.Count;
  for ImageNo:=0 to (Count-1) do begin
    s:=FileList.Items[ImageNo];
    LastPathDelim:=length(s);
    while (s[LastPathDelim]<>PATH_DELIM) do dec(LastPathDelim);
    with List[ImageNo] do begin
      FilePath:=copy(s,1,LastPathDelim);
      FileName:=copy(s,LastPathDelim+1,63);
      Selected:=false;
    end;
  end;
end;

procedure TSearchForm.BAcceptClick(Sender: TObject);
var Unsorted:array of TImgRec;
    cnt:integer;
begin
  cnt:=FileList.Items.Count;
  if cnt<1 then begin
    MainForm.ImageCount:=0;
    SetLength(MainForm.Images,0);
    ListForm.RefreshList;
    MainForm.Go(0);
  end else begin
    MainForm.ImageCount:=cnt;
    if CSort.Checked then begin
      SetLength(Unsorted,cnt);
      GetList(Unsorted);
      MainForm.SortList(Unsorted);
      SetLength(Unsorted,0);
    end else begin
      SetLength(MainForm.Images,cnt);
      GetList(MainForm.Images);
    end;
    with MainForm do begin
      CurrentImage:=FileList.ItemIndex;
      if (CurrentImage<0) OR (CurrentImage>=cnt) then CurrentImage:=0;
      ImageCountStr:='/'+IntToStr(ImageCount)+'] ';
      ListForm.RefreshList;
      Go(CurrentImage);
    end;
  end;
  ModalResult:=mrOK;
end;

procedure TSearchForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#27 then begin
    if Searching
      then SearchThread.Terminate
      else ModalResult:=mrCancel;
  end;
end;

end.
